/*_############################################################################
  _## 
  _##  SNMP4J-AgentJMX - JMXNotificationSupport.java  
  _## 
  _##  Copyright (C) 2006-2009  Frank Fock (SNMP4J.org)
  _##  
  _##  This program is free software; you can redistribute it and/or modify
  _##  it under the terms of the GNU General Public License version 2 as 
  _##  published by the Free Software Foundation.
  _##
  _##  This program is distributed in the hope that it will be useful,
  _##  but WITHOUT ANY WARRANTY; without even the implied warranty of
  _##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  _##  GNU General Public License for more details.
  _##
  _##  You should have received a copy of the GNU General Public License
  _##  along with this program; if not, write to the Free Software
  _##  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
  _##  MA  02110-1301  USA
  _##  
  _##########################################################################*/

package org.snmp4j.agent.mo.jmx;

import javax.management.Notification;
import javax.management.NotificationListener;
import java.util.HashMap;
import java.util.Map;
import org.snmp4j.smi.OID;
import org.snmp4j.smi.VariableBinding;
import org.snmp4j.agent.NotificationOriginator;

/**
 * The <code>JMXNotificationSupport</code> receives JMX notifications and
 * forwards them to a {@link NotificationOriginator}.
 *
 * @author Frank Fock
 * @version 1.0
 */
public class JMXNotificationSupport implements NotificationListener {

  private Map<OID,MBeanNotificationInfo> notificationInfos;
  private NotificationOriginator notificationOriginator;

  /**
   * Creates a <code>JMXNotificationSupport</code> to will use the supplied
   * <code>NotificationOriginator</code> to send SNMP notifications on behalf
   * of JMX notifications.
   *
   * @param notificationOriginator
   *   a <code>NotificationOriginator</code> instance
   */
  public JMXNotificationSupport(NotificationOriginator notificationOriginator) {
    this.notificationOriginator = notificationOriginator;
    notificationInfos = new HashMap<OID,MBeanNotificationInfo>();
  }

  /**
   * Adds a notification type to this support instance.
   * @param notificationID
   *   a notification or trap OID.
   * @param notificationInfo
   *   a <code>MBeanNotificationInfo</code> instance describing the mapping
   *   between JMX and SNMP notification.
   */
  public void add(OID notificationID, MBeanNotificationInfo notificationInfo) {
    notificationInfos.put(notificationID, notificationInfo);
  }

  /**
   * Adds all supplied notification types to this support instance.
   * @param notificationDefinitions
   *    an array containing zero or more arrays with two elements where the
   *    first is a notification or trap OID and the second element is
   *    a <code>MBeanNotificationInfo</code> instance describing the mapping
   *    between JMX and SNMP notification.
   */
  public void addAll(Object[][] notificationDefinitions) {
    for (Object[] tableDescr : notificationDefinitions) {
      MBeanNotificationInfo mBeanInfo =
          (MBeanNotificationInfo)tableDescr[1];
      notificationInfos.put((OID)tableDescr[0], mBeanInfo);
    }
  }

  /**
   * Invoked when a JMX notification occurs, this method sends a SNMP
   * notification
   *
   * @param notification
   *   The notification.
   * @param handback
   *   An opaque object which helps the listener to associate
   *   information regarding the MBean emitter. This object is passed to the
   *   MBean during the addListener call and resent, without modification, to
   *   the listener. The MBean object should not use or modify the object.
   */
  public void handleNotification(Notification notification, Object handback) {
    MBeanNotificationInfo info = notificationInfos.get(handback);
    if (info != null) {
      VariableBinding[] vbs =
          info.getNotificationPayload(notification.getUserData());
      notificationOriginator.notify(info.getContext(), (OID)handback, vbs);
    }
  }
}
